#include "complexe.hpp"



std::ostream & operator<<(std::ostream& flux, complexe const &z)
{
  /*
	Affichage "joli" d'un nombre complexe. On aurait pu ecrire
	un affichage plus rudimentaire avec
	flux << z.x << " + "<< z.y << " i "
  */

  if (z.x == 0 && z.y == 0)
	flux << 0;

  if (z.x != 0)
	flux << z.x;
  
  if (z.x!=0 && z.y>0)
	{
	  if (z.y==1)
		flux << " + i";
	  else
		flux << " + " << z.y << " i";
	}

  if (z.x==0 && z.y>0)
	{
	  if (z.y==1)
		flux << "i";
	  else
		flux << z.y << " i";
	}

  if (z.y<0)
	{
	  if (z.y==-1)
		flux << " - i";
	  else
		flux << " - " << -z.y << " i";
	}

  return flux;
}



complexe operator+(complexe const &z1, complexe const &z2)
{
  complexe z(z1);
  z += z2;
  return z;
}

complexe operator-(complexe const &z1, complexe const &z2)
{
  return z1 + (-z2);
}

complexe operator*(complexe const &z1, complexe const &z2)
{
  complexe z(z1);
  z *= z2;
  return z;
}

complexe operator/(complexe const &z1, complexe const &z2)
{
  complexe z(z1);
  z /= z2;
  return z;
}


/*
  Contrairement aux cinq fonctions precedentes, les quatre fonctions suivantes sont
  des methodes (c'est a dire des membre qui sont aussi des fonction) de la classe
  complexe. On doit donc le signaler au compilateur a l'aide de la syntaxe
  "complexe::".
  Hors de la declaration de la classe, ecrire "ma_classe::f" signifie que l'on
  definit la methode "f", membre de la classe "ma_classe".
  
*/


complexe &complexe::operator+=(complexe const &z)
{
  x += z.x;
  y += z.y;
  return *this;
}

complexe &complexe::operator*=(complexe const &z)
{
  double x_sauv = x;
  x = x*z.x - y*z.y;
  y = x_sauv*z.y + y*z.x;
  return *this;
}

complexe &complexe::operator-=(complexe const &z)
{
  x += z.x;
  y -= z.y;
  return *this;
}

complexe &complexe::operator/=(complexe const &z)
{
  // Le carre du module de z
  double nz = z.x*z.x + z.y*z.y;

  // on multiplie par 1/z (noter que 1/(a+ib) = (a-ib)/(a^2+b^2) )
  double x_sauv = x;
  x = (x*z.x + y*z.y) / nz;
  y = (y*z.x - x_sauv*z.y ) /nz;
  return *this;
}

