#ifndef _COMPLEXE_HPP_
#define _COMPLEXE_HPP_


#include <iostream>

class complexe
{
private:
  double x, y; // Membres de la classe


public:
  /*
	Constructeur : fonction qui "remplit" les champs x et y de la classe.
	Les constructeurs d'une classe sont des fonctions portant le meme nom
	que la classe et n'ayant pas de type de retour.
	
	La declaration suivante cree en fait non pas une mais trois fonctions :
	- un constructeur prenant deux arguments (la partie reelle et la partie
	  imaginaire du complexe considere)
	  
	- un constructeur prenant un argument, correspondant a la partie reelle.
	  La partie imaginaire sera initialisee a la valeur par defaut de 0.
	  En particulier, l'appel de complexe(3) va creer le nombres complexe 3+0i,
	  ce qui correspond bien a l'intuition qu'on voudrait avoir.
	  
	- un constructeur sans argument (et donc un constructeur par defaut), qui
	  cree le complexe 0.
  */
  complexe(double reelle=0, double imaginaire=0) : x(reelle), y(imaginaire) {}


  /*
	Aucun champ de la classe "complexe" n'est un pointeur. Il n'est donc pas
	necessaire de redefinir un constructeur par copie, un destructeur et un operateur=.
	Le compilateur cree automatiquement ces fonctions.
  */



  
  // Cette fonction permet d'afficher la partie reelle d'un nombre complexe, mais
  // cette partie reelle ne pourra pas etre modifiee(grace au mot cle const).
  double const& reel() const
  {return x;}

  // Meme chose pour la partie imaginaire.
  double const& imag() const
  {return y;}



  // Fonction qui a un complexe associe son conjugue.
  complexe conj() const
  {return complexe(x,-y);}
  

  // L'operateur <<, pour facilier l'affichage d'un nombre complexe. Cette
  // fonction est seulement declaree comme amie de la classe (et pas comme
  // methode) car la declaration comme methode obligerait a ecrire "z << std::cout"
  // pour afficher z dans le flux std::cout.
  friend std::ostream & operator<<(std::ostream& o, complexe const &z); // Ecrire "flux << z" reviendra a un appel de "operator<<(flux,z)"


  
  // Declaration et definition des operateurs + et - unaire (c'est a dire x -> x et x -> -x)
  complexe operator+(void) const
  {return *this;}

  complexe operator-(void) const // Ecrire "-z" correspondra a un appel de "z.operator-()"
  {return complexe(-x,-y);}


  
  // Declaration des operateurs binaires. Ces fonctions sont declarees comme amies plutot
  // que comme methodes car il n'y a pas de variable privilegiee dans une ecriture comme
  // x+y, x-y, x*y ou x/y
  friend complexe operator+(complexe const &z1, complexe const &z2); // Ecrire "z1 + z2" correspondra a un appel de "operator+(z1,z2)"
  friend complexe operator-(complexe const &z1, complexe const &z2);
  friend complexe operator*(complexe const &z1, complexe const &z2);
  friend complexe operator/(complexe const &z1, complexe const &z2);
  
  // Declaration des operateurs d'assignement. Ces operateurs-la sont declares comme membres car
  // il y a une dissymetrie entre les deux membre de x += y.
  complexe &operator+=(complexe const &z); // Ecrire "z1 += z2" correspondra a un appel de "z1.operator+(z2)"
  complexe &operator-=(complexe const &z);
  complexe &operator*=(complexe const &z);
  complexe &operator/=(complexe const &z);
  // Le type de retour des operateurs +=, -=, etc. est une *reference* sur complexe, car il
  // existe deja une variable qui a la bonne valeur (il s'agit de la variable dont on appelle
  // la methode). Ce n'etait pas le cas pour les operateurs +, -, *, /.
  
  
  

  /*
	Remarque : dans toutes les dernieres methode, les arguments sont appeles sous forme de
	references constante. En effet :
	- cela permet de ne pas recopier les arguments a chaque appel d'une fonction. En effet, pour
	  une classe plus evoluee (par exemple des matrices de grande taille), la recopie peut etre
	  couteuse en temps de calcul.
	  
	- le mot-cle const permet de s'assurer que l'argument ne sera pas modifie, comme si il etait
  	  passe par valeur.
  */
};


// On definit le complexe I avec I^2 = -1
const complexe I(0,1);


#endif /* _COMPLEXE_HPP_ */
